@@ -1,4 +1,4 @@
-from visidata import vd, Sheet, options, Column, asyncthread, Progress, PivotGroupRow, ENTER
+from visidata import vd, Sheet, options, Column, asyncthread, Progress, PivotGroupRow, ENTER, HistogramColumn
 
 from visidata.loaders._pandas import PandasSheet
 from visidata.pivot import PivotSheet
@@ -11,8 +11,8 @@ class DataFrameRowSliceAdapter:
     by visidata's selectRow implementation.
     """
     def __init__(self, df, mask):
-        import pandas as pd
-        import numpy as np
+        pd = vd.importExternal('pandas')
+        np = vd.importExternal('numpy')
         if not isinstance(df, pd.DataFrame):
             vd.fail('%s is not a dataframe' % type(df).__name__)
         if not isinstance(mask, pd.Series):
@@ -62,15 +62,15 @@ class DataFrameRowSliceIter:
         self.index += 1
         return row
 
+def makePandasFreqTable(sheet, *groupByCols):
+    fqcolname = '%s_freq' % '-'.join(col.name for col in groupByCols)
+    return PandasFreqTableSheet(sheet.name, fqcolname, groupByCols=groupByCols, source=sheet)
+
+
 class PandasFreqTableSheet(PivotSheet):
     'Generate frequency-table sheet on currently selected column.'
     rowtype = 'bins'  # rowdef FreqRow(keys, sourcerows)
 
-    def __init__(self, sheet, *groupByCols):
-        fqcolname = '%s_%s_freq' % (sheet.name, '-'.join(col.name for col in groupByCols))
-        super().__init__(fqcolname, groupByCols, [], source=sheet)
-        self.largest = 1
-
     def selectRow(self, row):
         # Select all entries in the bin on the source sheet.
         # Use the internally defined _selectByLoc to avoid
@@ -86,8 +86,7 @@ class PandasFreqTableSheet(PivotSheet):
     def updateLargest(self, grouprow):
         self.largest = max(self.largest, len(grouprow.sourcerows))
 
-    @asyncthread
-    def reload(self):
+    def loader(self):
         'Generate frequency table then reverse-sort by length.'
         import pandas as pd
 
@@ -95,8 +94,6 @@ class PandasFreqTableSheet(PivotSheet):
         # (e.g. as a histogram). We currently don't provide support for this
         # for PandasSheet, although we could implement it with a pd.Grouper
         # that operates similarly to pd.cut.
-        super().initCols()
-
         df = self.source.df.copy()
 
         # Implementation (special case): for one row, this degenerates
@@ -141,9 +138,7 @@ class PandasFreqTableSheet(PivotSheet):
                            getter=lambda col,row: len(row.sourcerows)),
                     Column('percent', type=float,
                            getter=lambda col,row: len(row.sourcerows)*100/df.shape[0]),
-                    Column('histogram', type=str,
-                           getter=lambda col,row: options.disp_histogram*(options.disp_histolen*len(row.sourcerows)//value_counts.max()),
-                           width=options.disp_histolen+2),
+                    HistogramColumn('histogram', type=str, width=self.options.default_width*2)
                     ]:
             self.addColumn(c)
 
@@ -164,6 +159,9 @@ class PandasFreqTableSheet(PivotSheet):
                 {}
             ))
 
+    def openRow(self, row):
+        return self.source.expand_source_rows(row)
+
 @Sheet.api
 def expand_source_rows(sheet, row):
     """Support for expanding a row of frequency table to underlying rows"""
@@ -171,16 +169,10 @@ def expand_source_rows(sheet, row):
         vd.fail("no source rows")
     return PandasSheet(sheet.name, vd.valueNames(row.discrete_keys, row.numeric_key), source=row.sourcerows)
 
-PandasSheet.addCommand('F', 'freq-col', 'vd.push(PandasFreqTableSheet(sheet, cursorCol))', 'open Frequency Table grouped on current column, with aggregations of other columns')
-PandasSheet.addCommand('gF', 'freq-keys', 'vd.push(PandasFreqTableSheet(sheet, *keyCols))', 'open Frequency Table grouped by all key columns on source sheet, with aggregations of other columns')
+PandasSheet.addCommand('F', 'freq-col', 'vd.push(makePandasFreqTable(sheet, cursorCol))', 'open Frequency Table grouped on current column, with aggregations of other columns')
+PandasSheet.addCommand('gF', 'freq-keys', 'vd.push(makePandasFreqTable(sheet, *keyCols))', 'open Frequency Table grouped by all key columns on source sheet, with aggregations of other columns')
 
-PandasFreqTableSheet.addCommand('t', 'stoggle-row', 'toggle([cursorRow]); cursorDown(1)', 'toggle selection of rows grouped in current row in source sheet')
-PandasFreqTableSheet.addCommand('s', 'select-row', 'select([cursorRow]); cursorDown(1)', 'select rows grouped in current row in source sheet')
-PandasFreqTableSheet.addCommand('u', 'unselect-row', 'unselect([cursorRow]); cursorDown(1)', 'unselect rows grouped in current row in source sheet')
-PandasFreqTableSheet.addCommand(ENTER, 'open-row', 'vd.push(source.expand_source_rows(cursorRow))', 'open copy of source sheet with rows that are grouped in current row')
+PandasFreqTableSheet.init('largest', lambda: 1)
+PandasFreqTableSheet.options.numeric_binning = False
 
-PandasFreqTableSheet.class_options.numeric_binning = False
-
-vd.addGlobals({
-        'PandasFreqTableSheet': PandasFreqTableSheet,
-        })
+vd.addGlobals(makePandasFreqTable=makePandasFreqTable)
