@@ -19,13 +19,16 @@ def isUndoableCommand(longname):
 def addUndo(vd, undofunc, *args, **kwargs):
     'On undo of latest command, call ``undofunc(*args, **kwargs)``.'
     if vd.options.undo:
-        # occurs when VisiData is just starting up
-        if getattr(vd, 'activeCommand', UNLOADED) is UNLOADED:
+        # occurs when VisiData is just starting up.
+        # very early in startup, modifyCommand does not yet exist
+        if not vd.activeCommand:
             return
         r = vd.modifyCommand
+        if not r or not isUndoableCommand(r.longname):
+            return
         # some special commands, like open-file, do not have an undofuncs set
         # do not set undofuncs for non-logged commands
-        if not r or not isUndoableCommand(r.longname) or not vd.activeCommand or not vd.isLoggableCommand(vd.activeCommand.longname):
+        if not vd.isLoggableCommand(vd.activeCommand):
             return
         if not r.undofuncs:
             r.undofuncs = []
@@ -37,8 +40,10 @@ def undo(vd, sheet):
     if not vd.options.undo:
         vd.fail("options.undo not enabled")
 
-    # don't allow undo of first command on a sheet, which is always the command that created the sheet.
-    for i, cmdlogrow in enumerate(sheet.cmdlog_sheet.rows[:0:-1]):
+    cmdlogrows = itertools.dropwhile(lambda r: r.longname == 'set-option', sheet.cmdlog_sheet.rows)
+    # skip the first remaining command, to exclude it from undo,
+    # because it is always the one that created the sheet
+    for i, cmdlogrow in enumerate(reversed(list(cmdlogrows)[1:])):
         if cmdlogrow.undofuncs:
             for undofunc, args, kwargs, in cmdlogrow.undofuncs[::-1]:
                 undofunc(*args, **kwargs)
